#!/usr/local/bin/perl -w
#***********************************************************************************#
# Name	   : MURALI K. KADIMI                                                       #
# Id	   : 641-64-8252                                                            #
# Course   : Data Mining                                                            #
# Details  : Implementation of Agrawal's Apriori algorithm for finding association  #
#            rules.                                                                 #
# Compile & Run : perl pgm1.pl <input filename>                                     #
#***********************************************************************************#

$MINSUP = 6;       # Minimum Support
$MINCONF = 0.6;    # Minimum Confidence

$file = "@ARGV";
open(INFO, $file);
@lines = <INFO>;   # Read the input file into an array
close(INFO);

# Attach tokens to elements such that the token identifies the column 
# to which the data point belongs 
$i = 0;
foreach $line (@lines) {
   chop $line;
   @words = split (/\s+/, $line);
   $words[0] = 'a!'.$words[0];
   $words[1] = 'b!'.$words[1];
   $words[2] = 'c!'.$words[2];
   $words[3] = 'd!'.$words[3];
   $words[4] = 'e!'.$words[4];
   $words[5] = 'f!'.$words[5];
   $words[6] = 'g!'.$words[6];
   $words[7] = 'h!'.$words[7];
   $line = "@words";
   $arrTran[$i] = "$line"."\n";  # put the converted array into another array
   ++$i;
}

# hash the elements of size 1
&hash_one(*arrTran, *hash);      # %hash is the hash table in with itemsets are stored

#load the itemset L1 into an array    
$i = 0;
while (($key, $val) = each(%hash)) {
   if ($val >= $MINSUP) {
      $array1[$i] = $key;
      ++$i;
   }
}
@array1 = sort(@array1);        # keep the array in sorted order 

# remove the elments which do not have MINSUP
&keep_MINSUP(*hash);

@arrt1 = ();
# generate candidate sets of size 2
$k = 0; 
for ($i = 0; $i < @array1 - 1; ++$i) {
   $arrt1[0] = $array1[$i];
   for ($j = $i + 1; $j < @array1; ++$j) {
      $arrt1[0] =~ /(^.)/;  
      $a1 = $1;
      $array1[$j] =~ /(^.)/;  
      $a2 = $1;
      # if the two elements do not belong to the same column the include the pair
      if ($a1 ne $a2) {
         $arrt1[1] = $array1[$j];  
         @arrt1 = sort(@arrt1);
         $arrC1[$k] = "@arrt1";   # store in a sorted order
         ++$k;
      }
   }
}  

@arrC1 = sort(@arrC1);
@arrC2 = ();

# Generate Large itemsets of size 2 into arrC2
&check_support_hash(*arrC1, *arrC2, *arrTran, *hash);
@arrC2 = sort(@arrC2);

@arrC3 = @arrC2;
while (1) {
   @arrTempC3 = ();
   $flg = 0;
   for ($i = 0; $i < @arrC3 - 1; ++$i) {
      if ($arrC3[$i] =~ /(.*)\s+(.)!(\d+)$/) {
         $x = $1;
         $y = $2;
         #$z = $2."!".$3;
      }
      else {
         print "no \n";
      }

      for ($j = $i + 1; $j < @arrC3; ++$j) {
         if ($arrC3[$j] =~ /$x/) {
            if ($arrC3[$j] =~ /(.*)\s+(.)!(\d+)$/) {
               if ($y ne $2) {
                  $a = $2."!".$3;
                  $b = $arrC3[$i]." ".$a;    # append a new element to the itemset

                  # check if the subsets of the above itemset belong to the L itemsets
                  @arrT = ();
                  @arrT = split (/\s+/, $b);
                  $v = pop(@arrT);
                  $u = pop(@arrT);
                  $w = $u." ".$v ;
                  $add_flg = 0;
                  foreach $wordC2 (@arrC2) {
                     if ($wordC2 =~ /$w/) {
                        $add_flg = 1;        
                        last;
                     }
                  }

                  if ($add_flg == 1) {
                     push(@arrTempC3, $b);   # add the itemsets which are not pruned 
                     $flg = 1;
                  }
               }
            }
         }
         #else { last; }
      }      
   }
   @arrC3 = ();

   #check for minmum support and insert into hash
   &check_support_hash(*arrTempC3, *arrC3, *arrTran, *hash);
   @arrC3 = sort(@arrC3);

   if ($flg == 0) {
      last;          # if no more Large itemsets then exit
   }
}

#&print_hash(*hash);
#print "\n";

foreach $key (keys %hash) {
   @tempWs = split (/\s+/, $key);
   $k = @tempWs;
   if ($k >=2 ) {       
      &gen_rules($key, $key); # generate rules for all large itemsets
   }
}

exit;
#***********************************************************************************#
# Main Program Ends Here                                                            #
#***********************************************************************************#


#***********************************************************************************#
# Subroutine to print the elements of hash table                                    #
#***********************************************************************************#
sub print_hash {
   local(*hash1) = @_;
   local($key1, $val1);

   while (($key1, $val1) = each(%hash1)) {
      if ($val1 >= $MINSUP) {
         print "$key1-$val1\n";      # if the sup >= MINSUP then output the value
      }
   }
   print "\n";
}


#***********************************************************************************#
# Subroutine to hash the elements of size 1                                         #
#***********************************************************************************#
sub hash_one {
   local(*arrTemp1, *hash1) = @_;
   local($line, $word, $newcount);
   local(@words);

   foreach $line (@arrTemp1) {
      @words = split (/\s+/, $line);
      foreach $word (@words) {
         if ($line =~ /$word/) {
            if (!$hash1{$word}) {
               $newcount = 0;
            }
            else {
               $newcount = $hash1{$word};
            }
            $hash1{$word} = $newcount + 1;
         }
      }
   }
}


#***********************************************************************************#
# Subroutine to check for support and then insert into hash table                   #
#***********************************************************************************#
sub check_support_hash {
   local(*arrTemp1, *arrTemp2, *arrTran1, *hash1) = @_;
   local($lineC1, $line, $word, $count, $flg1);
   local(@words);

   foreach $lineC1 (@arrTemp1) {
      @words = split (/\s+/, $lineC1);   
      $count = 0;
      foreach $line (@arrTran1) {         # arrTran1 points to the Trasactions
         $flg1 = 0;
         foreach $word (@words) {
            if ($line =~ /$word/) {
               $flg1 = 1;
            }
            else {
               $flg1 = 0;                 # if a mismatch is found the exit loop
               last;
            }
         }
         if ($flg1 == 1) {
            $count = $count + 1;          # count the support in the Transactions
         }
      }
      if ($count >= $MINSUP) {
         push(@arrTemp2, $lineC1);
         $hash1{$lineC1} = $count;        # if sup >= MINSUP then hash
      }
   }
}


#***********************************************************************************#
# Subroutine to remove all the elments of hash table                                #
#***********************************************************************************#
sub remove_all {
   local(*hash1) = @_;     
   local($key1);

   foreach $key1 (keys %hash1) {
      delete $hash1{$key1};
   }
}


#***********************************************************************************#
# Subroutine to remove the elements with sup < MINSUP from hash table               #
#***********************************************************************************#
sub keep_MINSUP {
   local(*hash1) = @_;     
   local($key1, $val1);

   while (($key1, $val1) = each(%hash1)) {
      if ($val1 < $MINSUP) {
         delete $hash1{$key1};    # delete element if sup < MINSUP
      }
   }
}


#***********************************************************************************#
# Subroutine to generate assoc rules (a recursive function)                         #
#***********************************************************************************#
sub gen_rules {
   local($val1, $val2) = @_;
   local($i, $line, $word, $tempL, $conf, $val3, $rhs, $n, $name1, $name2, $word1);
   local(@words, @arrA, @tempW, @words1);

   @arrA = ();
   $n = 0;
   @words = split (/\s+/, $val2);
   foreach $word (@words) {
      $tempL = $val2;
      $tempL =~ s/$word//;             # replace the word with space
      $tempL =~ s/^\s+(.)/$1/;
      @tempW = split (/\s+/, $tempL);
      $tempL = "@tempW";
      $n = @tempW;
      push(@arrA, $tempL);             # push the m-1 subset in to array
   }
   @arrA = sort(@arrA);                # arrA conatins the m-1 size subsets of lk.

   foreach $val3 (@arrA) {
      #print ": $val1 : $val3 :\n";
      $conf = $hash{$val1} / $hash{$val3};
      if ($conf >= $MINCONF) {
        $rhs = $val1;
        @words1 = split(/\s+/, $val3);
        foreach $word1 (@words1) {     # rhs = #val1 - $val3
           $rhs =~ s/$word1//;           
        } 
        $rhs =~ s/^\s+(.)/$1/;
        @tempW = split (/\s+/, $rhs);
        $rhs = "@tempW";
        $name1 = &convert_name($val3);
        $name2 = &convert_name($rhs);
        #print "$val3 => $rhs       ";   # print the rule
        print "$name1 => $name2       ";   # print the rule
        printf "(conf = %.2f, sup = %d)\n", $conf ,$hash{$val1};
        if($n > 1) {
           &gen_rules($val1, $val3);
        }
      }
   }
}


#***********************************************************************************#
# Subroutine to convert the tokens to Column Names                                  #
#***********************************************************************************#
sub convert_name {
   local($a) = @_;
   local($word, $newname, $b);
   local(@words, @arrName);

   @words = split (/\s+/, $a);
   foreach $word (@words) {
      $word =~ /(^.)/;
      $b = $1;
      if ($b eq 'a') {
         $word =~ s/^.!/origin=/; 
      }
      if ($b eq 'b') {
         $word =~ s/^.!/cylinders=/; 
      } 
      if ($b eq 'c') {
         $word =~ s/^.!/mpg=/; 
      } 
      if ($b eq 'd') {
         $word =~ s/^.!/accleration=/; 
      } 
      if ($b eq 'e') {
         $word =~ s/^.!/modelyear=/; 
      } 
      if ($b eq 'f') {
         $word =~ s/^.!/displacement=/; 
      } 
      if ($b eq 'g') {
         $word =~ s/^.!/horsepower=/; 
      } 
      if ($b eq 'h') {
         $word =~ s/^.!/weight=/; 
      } 
      push(@arrName, $word);
   }
   $" = ", ";
   $newname = "@arrName";
   $" = " ";
   return ($newname);
}


#***********************************************************************************#
#                                    END OF CODE                                    #
#***********************************************************************************#
